from pathlib import Path

from kal.base.json_obj import BaseJson
from kal import path
from kal.helper import dig


class Config(BaseJson):
    filepath = path.CONFIG_FILE

    @classmethod
    def get(cls, *keys, default=None):
        return dig(cls.data, *keys, default=default)

    @classmethod
    def link(cls, name):
        return cls.get('link', 'list', name, default=None)

    @classmethod
    def script(cls, name):
        return cls.get('script', 'list', name, default=None)

    @classmethod
    def script_root(cls):
        script_root = cls.get('script', 'root', default=None)
        if script_root:
            script_root = Path(script_root)
        return script_root


    @classmethod
    def link_root(cls):
        link_root = cls.get('link', 'root', default=None)
        if link_root:
            link_root = Path(link_root)
        return link_root

    @classmethod
    def docker(cls, name):
        return cls.get('docker', name, default=None)



from cleo import argument, option

from .base import Command
from kal.utils.github import url_parsing, clone


class CloneCommand(Command):
    name = 'clone'
    description = 'clone repository from github'
    arguments = [
        argument('repository', 'name of repository that you want to clone'),
        argument('target', 'target directory name', optional=True),
    ]
    options = [
        option('url', 'u', 'echo only url, not clone')
    ]

    def handle(self):
        clone_url = url_parsing(self.argument('repository'))
        if self.option('url'):
            self.line(clone_url)
            return 1
        clone(clone_url, self.argument('target'))



from .base import Command
from kal import path


class EnvCommand(Command):
    name = 'env'
    description = 'get kal envs'

    def handle(self):
        envs = {
            "SOURCE_DIR": path.SOURCE_DIR,
            "PROJECT_DIR": path.PROJECT_DIR,
            "USER_HOME_DIR": path.USER_HOME_DIR,
            "KAL_HOME_DIR": path.KAL_HOME_DIR,
            "DROPBOX_DIR": path.DROPBOX_DIR,
            "CONFIG_FILE": path.CONFIG_FILE,
            "STORAGE_FILE": path.STORAGE_FILE,
        }

        for name, env in envs.items():
            self.line("{}={}".format(name, env))



from cleo import argument

from .base import Command
from kal.utils.config import Config


class RunCommand(Command):
    name = 'run'
    description = 'Run docker container for local development'
    arguments = [
        argument('name', 'name of container'),
    ]

    def handle(self):
        name = self.argument('name')
        command = Config.docker(name)
        self.shell_call(command)


class PsCommand(Command):
    name = 'ps'
    description = 'show docker process managed by kal'

    def handle(self):
        self.shell_call('docker ps --filter name=\"kal-*\"')


class StopCommand(Command):
    name = 'stop'
    description = 'stop docker container'
    arguments = [
        argument('name', 'name of container'),
    ]

    def handle(self):
        name = self.argument('name')
        self.shell_call('docker stop kal-{}'.format(name))


class ListCommand(Command):
    name = 'list'
    description = 'all list of available containers'

    def handle(self):
        all_names = list(Config.get('docker', default={}).keys())
        self.line('\n'.join(all_names))


class DockerCommand(Command):
    name = 'docker'
    description = 'kal docker manager command'

    commands = [
        RunCommand(),
        PsCommand(),
        StopCommand(),
        ListCommand(),
    ]



from kal.helper import json_load, json_dump


class BaseJson:
    filepath = None
    default_data = {}
    data = None

    @classmethod
    def get_default_data(cls):
        return cls.default_data

    @classmethod
    def initialize(cls):
        if not cls.filepath.exists():
            json_dump(cls.get_default_data(), cls.filepath)
        with cls.filepath.open() as f:
            cls.data = json_load(f)

    @classmethod
    def get(cls, key, default=None):
        return cls.data.get(key, default)


class WritableJson(BaseJson):
    @classmethod
    def set(cls, key, value):
        cls.data[key] = value
        return value

    @classmethod
    def bulk_set(cls, **kwargs):
        for key, value in kwargs.items():
            cls.set(key, value)
        return True

    @classmethod
    def delete(cls, key):
        return cls.data.pop(key, None)

    @classmethod
    def save(cls):
        with cls.filepath.open('w') as f:
            json_dump(cls.data, f)



import json
from io import StringIO


def dig(obj, *keys, default=None):
    current_value = None
    index = 0
    try:
        for key in keys:
            if type(key) is str and key.isdigit():
                key = int(key)
            if index == 0:
                current_value = obj[key]
            else:
                current_value = current_value[key]
            index += 1
        return current_value
    except KeyError:
        return default
    except IndexError:
        return default


def json_load(some, **kwargs):
    kwargs['encoding'] = 'utf-8'

    if type(some) is str:
        try:
            return json.loads(some, **kwargs)
        except ValueError as e:
            with open(some) as f:
                return json.load(f, **kwargs)
    else:
        return json.load(some, **kwargs)


def json_dump(obj, target=None, indent=2, ensure_ascii=False, **kwargs):
    kwargs['indent'] = indent
    kwargs['ensure_ascii'] = ensure_ascii

    if target is not None:
        if not isinstance(target, StringIO):
            with open(target, 'w') as f:
                json.dump(obj, f, **kwargs)
        else:
            json.dump(obj, target, **kwargs)

    json.dumps(obj, **kwargs)



import pend
from cleo import Command as BaseCommand

from kal.utils import shell


class Command(BaseCommand):
    def shell_call(self, command, output=None):
        return shell.call(command, output=output)

    def line_error(self, text, style='error', verbosity=None):
        return super().line_error(text, style=style, verbosity=verbosity)


class TimeCommand(Command):
    mapper = ['hour', 'minute', 'second']

    def time_parse(self, time=None, from_tz='KST', target_tz=None):
        if time is None:
            time = self.argument('time')
        if target_tz is None:
            target_tz = self.argument('target_tz')

        time_arr = time.split(':')

        today = pend.today(from_tz)
        for index, v in enumerate(time_arr):
            today = today.replace(**{self.mapper[index]: int(v)})

        return today.in_timezone(target_tz)



from cleo import argument, option
from cleo.commands.base_command import CommandError

from kal.utils.config import Config
from .base import Command


class LinkCommand(Command):
    name = 'ln'
    description = 'link file creator'
    arguments = [
        argument('name', description='name of link in config json', optional=True),
    ]
    options = [
        option('list', 'l', description='show list of link config'),
        option('no-input', description='Run with no confirm')
    ]

    def generate_link(self, source, target):
        link_root = Config.link_root()
        if type(source) is not str or type(target) is not str:
            raise CommandError('Type error: source is not str type')

        if link_root and source[0] != '/':  # 루트에서 시작하면 그냥 넘어가기
            source_path = link_root / source
        else:
            source_path = source

        command = 'ln -sf {} {}'.format(
            source_path, target,
        )
        self.line(command)
        self.shell_call(command)
        return True

    def create_link(self, name, no_input=False):
        if name == 'all':
            link_config = Config.get('link', 'list', default=dict())
            if not no_input:
                if not self.confirm('전체 링크 작업을 실행하시겠습니까?', default=True):
                    self.line_error('실행 거부')
                    return False
            for name in link_config.keys():
                self.create_link(name, no_input=True)
            self.line('Done')
        else:
            link_path = Config.link(name)
            if not link_path:
                raise CommandError('link config not found: {}'.format(name))
            if not no_input:
                if not self.confirm('{} 링크 작업을 실행하시겠습니까?'.format(name), default=True):
                    self.line_error('실행 거부')
                    return False

            if type(link_path[0]) is str:
                if len(link_path) == 2:
                    return self.generate_link(link_path[0], link_path[1])
            elif type(link_path[0]) in (list, tuple, set):
                for i in link_path:
                    if len(i) != 2:
                        raise CommandError('Invalid format of link config: {}'.format(name))
                    self.generate_link(i[0], i[1])
                return True

            raise CommandError('Invalid format of link config: {}'.format(name))

    def add_list_row(self, table, name, config):
        if len(config) != 2:
            table.add_row([name, '<error>Invalid</error>', '<error>Invalid</error>'])
        else:
            table.add_row([name, *config])

    def handle(self):
        name = self.argument('name')

        if name:
            self.create_link(name, self.option('no-input'))
        elif self.option('list'):
            link_config = Config.get('link', 'list', default=dict())
            table = self.table()
            table.set_header_row(['name', 'source', 'target'])
            for name, config in link_config.items():
                if not config:
                    table.add_row([name, '<error>Invalid</error>', '<error>Invalid</error>'])
                if type(config[0]) in (list, tuple, set):
                    self.add_list_row(table, name, config[0])
                    for t in config[1:]:
                        self.add_list_row(table, '', t)
                elif type(config[0]) is str:
                    self.add_list_row(table, name, config)
                else:
                    table.add_row([name, '<error>Invalid</error>', '<error>Invalid</error>'])
            table.render(self.io)
        else:
            self.line_error('올바른 명령어를 입력해주세요.')



from kal.utils.config import Config
from kal.base.json_obj import WritableJson
from kal import path


class Storage(WritableJson):
    filepath = path.STORAGE_FILE

    @classmethod
    def get_default_data(cls):
        default_data = cls.default_data
        default_data.update(Config.get('storage', 'default', default=dict()))
        return default_data



import os
from pathlib import Path


SOURCE_DIR = Path(__file__).parent.absolute()
PROJECT_DIR = SOURCE_DIR.parent
USER_HOME_DIR = Path(os.getenv('HOME'))

if os.getenv('DROPBOX_PATH'):
    DROPBOX_DIR = Path(os.getenv('DROPBOX_PATH'))
else:
    DROPBOX_DIR = USER_HOME_DIR / 'Dropbox'

if os.getenv('KAL_HOME'):
    KAL_HOME_DIR = Path(os.getenv('KAL_HOME'))
else:
    KAL_HOME_DIR = USER_HOME_DIR / '.kal'

CONFIG_FILE = KAL_HOME_DIR / 'config.json'
STORAGE_FILE = KAL_HOME_DIR / 'storage.json'



import subprocess


def call(command, output=None, **kwargs):
    if output is not None:
        output = getattr(subprocess, output.upper())

    return subprocess.run(command, shell=True, stdout=output, encoding='utf-8')



from cleo import option, argument
from cleo.commands.base_command import CommandError

from kal.utils.ocel import Ocel
from .base import Command


class OcelCommand(Command):
    name = 'ocel'
    description = 'ocel command'
    arguments = [
        argument('template', 'name of template', optional=True),
        argument('target', 'target path', optional=True)
    ]
    options = [
        option('list', 'l', 'list of templates'),
        option('update', 'u', 'update ocel from github', )
    ]

    def handle(self):
        ls = self.option('list')
        update = self.option('update')
        template = self.argument('template')
        target = self.argument('target')
        ocel = Ocel()

        if template and (ls or update):
            raise CommandError('Invalid format of command')

        if template:
            ocel.run(template, target)
            self.line('Finished.')
        elif ls:
            template_list = ocel.template_list(True)
            table = self.table(['Name', 'Desc'])
            for name, meta in template_list.items():
                table.add_row([name, meta.get('desc', '')])
            table.render(self.io)
        elif update:
            ocel.update()
        else:
            raise CommandError('Invalid format of command')



from cleo import argument

from .base import TimeCommand


class KSTCommand(TimeCommand):
    name = 'kst'
    description = 'convert KST time to target timezone'
    arguments = [
        argument('time', 'target time to change'),
        argument('tz', 'target timezone to change', default='UTC', optional=True)
    ]

    def handle(self):
        time = self.argument('time')
        tz = self.argument('tz')
        parsed = self.time_parse(time, 'Asia/Seoul', tz)
        self.line("{} in {} is {}".format(time, tz, parsed.strftime('%H:%M')))


class UTCCommand(TimeCommand):
    name = 'utc'
    description = 'convert UTC time to target timezone'
    arguments = [
        argument('time', 'target time to change'),
        argument('tz', 'target timezone to change', default='Asia/Seoul', optional=True)
    ]

    def handle(self):
        time = self.argument('time')
        tz = self.argument('tz')
        parsed = self.time_parse(time, 'UTC', tz)
        self.line("{} in {} is {}".format(time, tz, parsed.strftime('%H:%M')))



from cleo import argument, option
from cleo.commands.base_command import CommandError

from kal.utils.config import Config
from .base import Command


class RunCommand(Command):
    name = 'run'
    description = 'run script in config'
    arguments = [
        argument('name', 'script name', optional=True),
        argument('exec', 'executer of script', optional=True)
    ]
    options = [
        option('list', 'l', 'list all runnable scripts')
    ]

    default_executer = '"${SHELL}"'

    def get_executer(self, name):
        runner = self.argument('exec')
        if runner:
            return runner
        config = Config.script(name)
        runner = config.get('exec')
        return runner or self.default_executer

    def run_script(self, name):
        config = Config.script(name)
        script_root = Config.script_root()
        if not config:
            raise CommandError('No such script: {}'.format(name))
        file = config.get('file')
        if not file:
            raise CommandError('Script does not have file')

        if script_root and file[0] != '/':
            file_path = script_root / file
        else:
            file_path = file

        runner = self.get_executer(name)
        command = '{} {}'.format(runner, file_path)
        self.line('Run command: {}'.format(command))
        self.shell_call(command)
        return True

    def full_file_path(self, file):
        script_root = Config.script_root()
        if script_root and file[0] != '/':
            file_path = script_root / file
        else:
            file_path = file

        return str(file_path)

    def handle(self):
        name = self.argument('name')

        if name:
            self.run_script(name)
        elif self.option('list'):
            script_config = Config.get('script', 'list', default=dict())
            table = self.table()
            table.set_header_row(['name', 'desc', 'script file'])
            for name, config in script_config.items():
                if not config:
                    table.add_row([name, '<error>Invalid</error>', '<error>Invalid</error>'])
                file = config.get('file', '')
                filepath = '' if not file else self.full_file_path(file)
                desc = config.get('desc', '')
                table.add_row([name, desc, filepath])
            table.render(self.io)
        else:
            self.line_error('올바른 명령어를 입력해주세요.')



import json
from pathlib import Path

from cookiecutter.main import cookiecutter

from kal import path
from kal.utils.config import Config
from kal.utils import github, shell


class Ocel:
    @property
    def default_path(self):
        return path.KAL_HOME_DIR / 'ocel'

    def __init__(self):
        ocel_path = Config.get('ocel', 'path')
        self.ocel_path = Path(ocel_path) if ocel_path else self.default_path
        if not self.ocel_path.exists():
            self.clone_ocel()

    def clone_ocel(self):
        github.clone(
            github.url_parsing('ocel'),
            target=self.ocel_path
        )

    def template_list(self, with_meta=False):
        if with_meta:
            result = {}
        else:
            result = []

        for f in self.ocel_path.glob('./*'):
            subdir = self.ocel_path / f
            if not subdir.is_dir():
                continue

            ocel_file = subdir / 'ocel.json'
            if not ocel_file.exists():
                continue

            if with_meta:
                with ocel_file.open() as file:
                    result[f.name] = json.load(file)
            else:
                result.append(f)

        return result

    def run(self, template, target=None):
        template_path = self.ocel_path / template
        output_dir = target or '.'
        return cookiecutter(str(template_path.absolute()), output_dir=output_dir)

    def update(self):
        commands = [
            'cd {}'.format(self.ocel_path),
            'git pull origin master'
        ]
        shell.call('&&'.join(commands))
        return True



from cleo import Application as BaseApplication

from kal import __version__
from kal.initialize import init
from . import commands as cmds


class Application(BaseApplication):
    def __init__(self, *args, **kwargs):
        init()
        super().__init__('kal', __version__)
        for command in self.get_default_commands():
            self.add(command)

    def get_default_commands(self):
        return [
            cmds.EnvCommand(),
            cmds.CloneCommand(),
            cmds.KSTCommand(),
            cmds.UTCCommand(),
            cmds.BoxCommand(),
            cmds.LinkCommand(),
            cmds.RunCommand(),
            cmds.DockerCommand(),
            cmds.OcelCommand(),
        ]



import pend

from .base import Command
from kal import path


class BoxCommitCommand(Command):
    name = 'commit'
    description = 'Dropbox git commit and push'

    def handle(self):
        commands = [
            'cd {}'.format(path.DROPBOX_DIR),
            'git add -A',
            'git commit -m "Dropbox update {}. Coomit by kal"'.format(pend.now().strftime('%Y-%m-%d %H:%M:%S')),
            'git push origin master'
        ]

        self.shell_call(' && '.join(commands))


class BoxCommand(Command):
    name = 'box'
    description = 'Dropbox helper command'
    commands = [
        BoxCommitCommand()
    ]

    def handle(self):
        self.line_error('하위 명령어를 입력하세요')
        return 1



from . import shell
from kal.utils.config import Config


def url_parsing(repository_name, protocol='ssh'):
    split_names = repository_name.split('/')

    if len(split_names) == 1:
        username = Config.get('github', 'username')
        repository_name = "{}/{}".format(username, repository_name)
    else:
        username = split_names[0]
        repository_name = "{}/{}".format(username, "-".join(split_names[1:]))

    if repository_name.find('.git') < 0:
        repository_name += '.git'

    if protocol == 'ssh':
        repository_name = 'git@github.com:{}'.format(repository_name)
    elif protocol == 'https':
        repository_name = 'https://github.com/{}'.format(repository_name)
    else:
        raise ValueError('Invalid protocol name: {}'.format(protocol))

    return repository_name


def clone(url, target=None):
    command = 'git clone {}'.format(url)
    if target:
        command += ' {}'.format(target)

    return shell.call(command)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kal', 'kal.base', 'kal.commands', 'kal.utils']

package_data = \
{'': ['*']}

install_requires = \
['cleo>=0.8.1,<0.9.0', 'cookiecutter>=1.7.2,<2.0.0', 'pend>=1.0.2,<2.0.0']

entry_points = \
{'console_scripts': ['kal = kal.runner:main']}

setup_kwargs = {
    'name': 'kal',
    'version': '0.2.1',
    'description': 'Personal assistant cli tool',
    'long_description': None,
    'author': 'Jrog',
    'author_email': 'jrog612@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from .clone import CloneCommand
from .time import KSTCommand, UTCCommand
from .box import BoxCommand
from .link import LinkCommand
from .docker import DockerCommand
from .ocel import OcelCommand
from .env import EnvCommand
from .run import RunCommand



import os

from kal.utils.config import Config
from kal import path
from kal.utils.storage import Storage


def init():
    if not path.KAL_HOME_DIR.exists():
        os.makedirs(path.KAL_HOME_DIR)
    Config.initialize()
    Storage.initialize()
